/*-------------------------------------------------------------------------+
| Copyright 2011 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.ext.quality.service;

import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.tooling.base.model.element.IHierarchicElement;
import org.fortiss.tooling.kernel.service.base.IEObjectAware;

/**
 * Interface for the migration provider implementations.
 * <P>
 * migration provider extensions are handled by {@link IModelQualityService}.
 * 
 * @author blaschke
 */
public interface IModelQualityProvider<C extends EObject> extends IEObjectAware<EObject> {

	/** Applies the IMetricProvider to the given model element. */
	void apply(C element);

	/**
	 * @return a simple example string demonstrating some metric capability
	 */
	String getExampleString();

	/**
	 * @param element
	 *            the element for which the matching {@link IModelQualityProvider} is searched
	 * @return the matching provider
	 */
	IModelQualityProvider<C> getProvider(IHierarchicElement element);

	/**
	 * Visits all nodes of the provider tree in post-order
	 * 
	 * @param visitor
	 *            the object on which the methods will be called
	 */
	void traverse(IVisitorModelQualityProvider visitor);

	/**
	 * @return set of keys of metrics contained in this provider
	 */
	List<String> metricKeys();

	/**
	 * Visitor pattern class
	 * 
	 * @author groh
	 */
	public interface IVisitorModelQualityProvider {

		/**
		 * @param provider
		 *            gets a HierarchicElementSizeProvider
		 */
		void accept(HierarchicElementSizeProvider provider);
	}
}
